__extensions [  "sound.jar" ]

breeds [ 
  players ;; the players move across the screen and play notes
  notes   ;; half or quarter notes
  cursor  ;; the gray note places notes on the staff 
  marks   ;; vertical lines break up measures
]
players-own [ pitch ]
notes-own [ duration ]
patches-own [ staff? ]
globals [ current-note current-beat x-offset y-offset scale note-names ]

to setup
  ca
  set scale (list 60 62 64 65 67 69 71 72)
  set note-names (list "C" "D" "E" "F" "G" "A" "B" "C")
  set x-offset -12
  set y-offset -3
  
  ask patches [ set staff? false ]
  
  ;; create the players
  create-custom-players 8 [
    ht
    set xcor x-offset
    set ycor who + y-offset  
    set pitch item who scale
    set heading 0
    set color black
    set shape "verticalline"
    set plabel-of patch-here item who note-names
    ask patches ;; every patch in my row is staff
      with [ pycor = ycor-of myself and pxcor >= xcor-of myself ] 
        [ set staff? true ]
    st
  ]
  
  ask turtles [ rt 90 ]

  ask patches [ set plabel-color grey ]
  
  ;; alternate white and gray lines
  ask patches with [ pycor mod 2 = 1 ] [ set pcolor white ]
  ask patches with [ pycor mod 2 = 0 ] [ set pcolor 9 ]
  
  ;; make patches outside the staff black
  ask patches with [ not staff? ] [ 
    set pcolor black 
  ]

  ;; create marks to indicate measures
  ask patches with [ staff? ] [
    if pxcor mod 4 = 0 [
      sprout 1 [ set breed marks set shape "verticalline" set color grey ]
    ]
  ]

  ;; create the cursor
  create-custom-cursor 1 [
    set color grey
    set label-color grey
    ht
  ]
end

to play
  every quarter-note-length [
    if metronome? [ play-drum "Hi Wood Block" 80 ]
    ask players [
      if ( any? notes-here )  [ 
        play-note 
          (item instrument-number instruments) 
          pitch 
          120
          ( quarter-note-length * duration-of one-of notes-here) ]
      fd 1
    ]
  ]
end

to draw-notes 

    ifelse ( staff?-of patch-at mouse-xcor mouse-ycor ) [
      ask cursor [ 
        st
        setxy mouse-xcor mouse-ycor 
        set shape note-shape
      ]
      set current-note item ( round mouse-ycor - y-offset ) note-names
      set current-beat round mouse-xcor - x-offset
      ask patches [ set plabel-color grey ]
      ask patches with [ ( round mouse-ycor - y-offset ) > 0 and plabel = item ( round mouse-ycor - y-offset ) note-names ] 
        [ set plabel-color black ] ;; highlight the note name label
    ]
    [
      ask cursor [ ht ]
    ]
    if mouse-down? [
      if not any? notes-on patch-at mouse-xcor mouse-ycor [
        create-custom-notes 1 [
          setxy round mouse-xcor round mouse-ycor
          set color black
          set shape note-shape
          if note-length = "1/4" [ set duration 1 ]
          if note-length = "1/2" [ set duration 2 ]
        ]
      ]
    ]
end

to erase-notes 
  ifelse ( staff?-of patch-at mouse-xcor mouse-ycor ) [
    ask cursor [ 
      st
      setxy mouse-xcor mouse-ycor 
      set shape "X"
      set current-note item ( round mouse-ycor - y-offset ) note-names
      set current-beat round mouse-xcor - x-offset
    ]
  ]
  [
    ask cursor [ ht ]
  ]
  if mouse-down? [
    ask notes-on patch-at mouse-xcor mouse-ycor [
      die
    ]
  ]
end

to-report note-shape 
  if note-length = "1/4" [ report "quarternote" ]
  if note-length = "1/2" [ report "halfnote" ]
end


; (C) 2004 Uri Wilensky.  This code may be freely copied, distributed,
; altered, or otherwise used by anyone for any purpose.
; 
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
; OWNERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
@#$#@#$#@
GRAPHICS-WINDOW
6
10
641
266
12
4
25.0
1
18
1
1
1

CC-WINDOW
6
455
641
546
Command Center

BUTTON
7
293
85
326
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
373
319
460
352
NIL
play
T
1
T
OBSERVER
T

BUTTON
7
365
112
398
NIL
draw-notes
T
1
T
OBSERVER
T

CHOICE
7
402
218
447
note-length
note-length
"1/4" "1/2"
0

SLIDER
475
410
623
443
quarter-note-length
quarter-note-length
0
5
0.15
0.05
1
NIL

BUTTON
116
365
222
398
NIL
erase-notes
T
1
T
OBSERVER
T

SWITCH
488
359
622
392
metronome?
metronome?
0
1
-1000

BUTTON
290
318
368
351
rewind
ask players [ \n  set xcor x-offset + 1\n]
NIL
1
T
OBSERVER
NIL

SLIDER
291
358
463
391
instrument-number
instrument-number
0
127
57
1
1
NIL

MONITOR
290
398
462
447
instrument-name
item instrument-number instruments
3
1

TEXTBOX
8
273
192
291
1. Click setup to draw a new staff

TEXTBOX
9
343
138
361
2. Draw some music

TEXTBOX
290
284
380
302
3. Play music

BUTTON
503
287
588
320
dance
 ask notes [ \n fd .02 \n rt (random 4) - 2\n if (random 10 < random 2) [\n rt 180]\n ]
T
1
T
OBSERVER
T

@#$#@#$#@
WHAT IS IT?
-----------
Composer is a toy model that lets you write songs and play them using the NetLogo sound extension.


HOW TO USE IT
-------------
Click SETUP to draw a staff.  Then choose a note length (quarter notes or half notes), click on DRAW-NOTES and draw notes on the staff.  Pick an instrument, click PLAY and listen to your music.


EXTENDING THE MODEL
-------------------
Add eighth notes.  Add sharps and flats.  Support several instruments at once. 
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

halfnote
true
0
Circle -7566196 true true 104 181 94
Line -7566196 true 196 230 194 26
Line -7566196 true 194 26 194 27
Rectangle -7566196 true true 183 31 195 212
Rectangle -7566196 true true 183 28 195 34
Circle -1 true false 117 196 68

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

quarternote
true
0
Circle -7566196 true true 104 181 94
Line -7566196 true 196 230 194 26
Line -7566196 true 194 26 194 27
Rectangle -7566196 true true 183 31 195 212
Rectangle -7566196 true true 183 28 195 34

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

verticalline
false
0
Rectangle -7566196 true true 1 2 15 377

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

x
false
0
Line -7566196 true 3 4 315 315
Line -7566196 true 296 4 -37 336
Polygon -7566196 true true 287 4 298 14 16 298 4 285
Polygon -7566196 true true 13 4 0 15 286 297 297 285

@#$#@#$#@
NetLogo 2.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
