globals [clock]

to setup
  ca
  set clock 0
  ask patches  ;; randomly set the patches' colors
    [ set pcolor ((random colors) * 10) + 5
        if pcolor = 75  ;; 75 is too close to another color so change it to 125
          [ set pcolor 125 ] ]
  do-plots
end

to go
  ask patches [match]
  set clock clock + 1
  do-plots
  if (variance values-from patches [pcolor]) = 0
    [ stop ]
end

to match       ;; patch procedure - each patch randomly picks
               ;; a neighboring patch to copy a color from
  set pcolor pcolor-of random-one-of neighbors
end

to do-plots 
  if not plot-groups? [ stop ]
  set-current-plot "Patch Colors" 
  set-current-plot-pen "color5" 
  plot  count patches with [pcolor = 5]
  set-current-plot-pen "color15" 
  plot  count patches with [pcolor = 15]
  set-current-plot-pen "color25" 
  plot  count patches with [pcolor = 25]
  set-current-plot-pen "color35" 
  plot  count patches with [pcolor = 35]
  set-current-plot-pen "color45" 
  plot  count patches with [pcolor = 45]
  set-current-plot-pen "color55" 
  plot  count patches with [pcolor = 55]
  set-current-plot-pen "color65" 
  plot  count patches with [pcolor = 65]
  set-current-plot-pen "color125" 
  plot  count patches with [pcolor = 125]
  set-current-plot-pen "color85" 
  plot  count patches with [pcolor = 85]
  set-current-plot-pen "color95" 
  plot  count patches with [pcolor = 95]
end 


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo GenDrift P local model.
; http://ccl.northwestern.edu/netlogo/models/GenDriftPlocal.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/GenDriftPlocal
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
235
10
555
351
15
15
10.0
0
10
1
1
1

CC-WINDOW
6
192
221
346
Command Center

BUTTON
94
50
162
83
go
go
T
1
T
OBSERVER

BUTTON
6
50
78
83
setup
setup
NIL
1
T
OBSERVER

SLIDER
7
97
99
130
colors
colors
2
10
5
1
1
NIL

PLOT
7
354
572
566
Patch Colors
Time
Number
0.0
100.0
0.0
200.0
true
true
PENS
"color5" 1.0 0 -7566196 true
"color15" 1.0 0 -65536 true
"color25" 1.0 0 -44544 true
"color35" 1.0 0 -6524078 true
"color45" 1.0 0 -256 true
"color55" 1.0 0 -11352576 true
"color65" 1.0 0 -16711936 true
"color125" 1.0 0 -65281 true
"color85" 1.0 0 -16711681 true
"color95" 1.0 0 -16745473 true

SWITCH
9
138
133
171
plot-groups?
plot-groups?
0
1
-1000

MONITOR
140
134
213
183
clock
round clock
0
1

@#$#@#$#@
WHAT IS IT? 
----------- 
This model is an example of localized random selection.  It shows that patches that randomly exchange colors converge on a single color. The idea, explained in more detail in Dennett's "Darwin's Dangerous Idea", is that trait drifts can occur without any particular purpose or 'selecting pressure'. 
 
The model starts with a random distribution of colors.  Each patch randomly picks a patch within its local five-patch neighborhood (to its immediate top, bottom, right, or left, or itself) to copy the color from, and changes itself to that patch's color.  After enough turns, a color will gain a slight dominance. By statistical advantage, and the fact that a color can become "extinct", a dominant color becomes more likely to win the entire grid.  However, because the process is random, there will usually be a series of dominant colors before one finally wins.  

HOW TO USE IT: 
-------------- 
The "setup" button intializes the model. 
The "go" button starts it, and runs continuously. 
Use the "colors" slider to select the number of colors competing. 
Use "plot-groups?" switch to turn plotting off if you want the model to run really fast.  But note that the "time" scale on the plot will no longer be accurate.

THINGS TO NOTICE: 
---------------- 
Notice that often colors that start with a higher initial number fail to win the grid. 

EXTENDING THE MODEL: 
-------------------- 
Are there variations on the local rule that patches use to re-color themselves that lead to other results? 

RELATED MODELS: 
--------------- 
GenDrift (P global)
GenDrift (T interact)
GenDrift (T reproduce)

CREDITS AND REFERENCES:
-----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo GenDrift P local model. http://ccl.northwestern.edu/netlogo/models/GenDriftPlocal. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2001 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/GenDriftPlocal for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

@#$#@#$#@
NetLogo 2.0alpha1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
