turtles-own [ energy ]
breeds [rabbits]

to setup
  ca
  grow-grass-and-weeds
  set-default-shape rabbits "rabbit-shape"
  create-custom-rabbits number [
    set color white
    randomize-position
    set energy random 10  ;start with a random amt. of energy
  ]
  setup-plot
  do-plot
end

to go
  grow-grass-and-weeds
  move
  do-plot
  if not any? turtles
    [ stop ]
end

to move
  ask rabbits
  [ step
    eat-grass
    eat-weeds
    reproduce
    death ]
end

to grow-grass-and-weeds
  ask patches [
    if pcolor = black [
      if ( random-float 1000 ) < weeds-grow-rate
        [ set pcolor violet ]
      if ( random-float 1000 ) < grass-grow-rate
        [ set pcolor green ]
  ] ]
end

to step   ;moving takes some energy
  rt random-float 50
  lt random-float 50
  fd 1
  set energy ( energy - 0.5 )
end

to eat-grass       ;;gain "grass-energy" by eating grass
  if ( pcolor = green )
  [ stamp black
    set energy ( energy + grass-energy ) ]
end

to eat-weeds     ;;gain "weed-energy" by eating weeds
  if ( pcolor = violet )
  [ stamp black
    set energy ( energy + weed-energy )]
end

to reproduce     ;;give birth to a new rabbit, but it takes lots of energy
  if ( energy > birth-threshold )
    [ set energy ( energy / 2 )
      hatch 1 [ fd 1 ] ]
end

to death     ;;die if you run out of energy
  if ( energy < 0 )
    [ die ]
end

to do-plot
  set-current-plot "Populations"
  set-current-plot-pen "grass"
  plot ( ( count patches with [pcolor = green] ) / 4 )
  set-current-plot-pen "rabbits"
  plot count turtles
  set-current-plot-pen "weeds"
  plot ( ( count patches with [pcolor = violet] ) / 4 )
end

to randomize-position
  setxy random-float screen-size-x
        random-float screen-size-y
end

to setup-plot ;; set up plotting
  set-current-plot "Populations"
  set-plot-y-range 0 number
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo Rabbits Grass Weeds model.
; http://ccl.northwestern.edu/netlogo/models/RabbitsGrassWeeds.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/RabbitsGrassWeeds
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
296
13
740
478
15
15
14.0
1
10
1
1
1

CC-WINDOW
3
373
274
524
Command Center

BUTTON
76
42
131
75
setup
setup
NIL
1
T
OBSERVER
T

BUTTON
141
42
196
75
go
go
T
1
T
OBSERVER
T

SLIDER
3
108
141
141
grass-grow-rate
grass-grow-rate
0.0
20.0
15.0
1.0
1
NIL

SLIDER
3
141
150
174
weeds-grow-rate
weeds-grow-rate
0.0
20.0
0.0
1.0
1
NIL

SLIDER
143
108
274
141
grass-energy
grass-energy
0.0
10.0
5.0
0.5
1
NIL

SLIDER
152
141
274
174
weed-energy
weed-energy
0.0
10.0
0.0
0.5
1
NIL

SLIDER
3
76
112
109
number
number
0.0
500.0
100.0
1.0
1
NIL

SLIDER
114
76
274
109
birth-threshold
birth-threshold
0.0
20.0
15.0
1.0
1
NIL

PLOT
3
176
274
372
Populations
Time
Pop
0.0
100.0
0.0
111.0
true
true
PENS
"grass" 1.0 0 -11352576 true
"rabbits" 1.0 0 -65536 true
"weeds" 1.0 0 -8716033 true

MONITOR
300
490
389
539
count rabbits
count rabbits
1
1

@#$#@#$#@
WHAT IS IT?
-----------
This project explores a simple ecosystem made up of rabbits, grass, and weeds. The rabbits wander around randomly, and the grass and weeds grow randomly.   When a rabbit bumps into some grass or weeds, it eats the grass and gains energy. If the rabbit gains enough energy, it reproduces. If it doesn't gain enough energy, it dies.

The grass and weeds can be adjusted to grow at different rates and give the rabbits differing amounts of energy.  The model can be used to explore the competitive advantages of these variables.


HOW TO USE IT
-------------
Click the SETUP button to setup the rabbits (red), grass (green), and weeds (violet). Click the GO button to start the simulation.

The NUMBER slider controls the initial number of rabbits. The BIRTH-THRESHOLD slider sets the energy level at which the rabbits reproduce.  The GRASS-GROWTH-RATE slider controls the rate at which the grass grows.  The WEEDS-GROWTH-RATE slider controls the rate at which the weeds grow.

The model's default settings are such that at first the weeds are not present (weeds-grow-rate = 0, weeds-energy = 0).  This is so that you can look at the interaction of just rabbits and grass.  Once you have done this, you can start to add in the effect of weeds.


THINGS TO NOTICE
----------------
Watch the COUNT-RABBITS monitor and the POPULATIONS plot window to see how the rabbit population changes over time. At first, there is not enough grass for the rabbits, and many rabbits die. But that allows the grass to grow more freely, providing an abundance of food for the remaining rabbits. The rabbits gain energy and reproduce. The abundance of rabbits leads to a shortage of grass, and the cycle begins again.

The rabbit population goes through a damped oscillation, eventually stabilizing in a narrow range. The total amount of grass also oscillates, out of phase with the rabbit population.

These dual oscillations are characteristic of predator-prey systems. Such systems are usually described by a set of differential equations known as the Lotka-Volterra equations. NetLogo provides a new way of studying predatory-prey systems and other ecosystems.


THINGS TO TRY
-------------
Leaving other parameters alone, change the grass-grow-rate and let the system stabilize again.  Would you expect that there would now be more grass?  More rabbits?

Change only the birth-threshold of the rabbits.  How does this affect the steady-state levels of rabbits and grass?

With the current settings, the rabbit population goes through a damped oscillation. By changing the parameters, can you create an undamped oscillation? Or an unstable oscillation?

In the current version, each rabbit has the same birth-threshold. What would happen if each rabbit had a different birth-threshold? What if the birth-threshold of each new rabbit was slightly different from the birth-threshold of its parent? How would the values for birth-threshold evolve over time?

Now add weeds by making the sliders WEEDS-GROW-RATE the same as GRASS-GROW-RATE and WEEDS-ENERGY the same as GRASS-ENERGY.  Notice that the amount of grass and weeds is about the same.

Now make grass and weeds grow at different rates.  What happens?

What if the weeds grow at the same rate as grass, but they give less energy to the rabbits when eaten (WEEDS-ENERGY is less than GRASS-ENERGY)?

Think of other ways that two plant species might differ and try them out to see what happens to their relative populations.  For example, what if a weed could grow where there was already grass, but grass couldn't grow where there was a weed?  What if the rabbits preferred the plant that gave them the most energy?

Run the model for a bit, then suddenly change the birth-threshold to zero.  What happens?


NETLOGO FEATURES
----------------
Notice that every black patch has a random chance of growing grass or
weeds each turn, using the rule:
  if (random 1000) < weeds-grow-rate and pcolor = black
    [set pcolor violet]
  if (random 1000) < grass-grow-rate and pcolor = black
    [set pcolor green]


RELATED MODELS
--------------
Look at Wolf Sheep Predation for another interacting ecosystem with different rules.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Rabbits Grass Weeds model. http://ccl.northwestern.edu/netlogo/models/RabbitsGrassWeeds. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/RabbitsGrassWeeds for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

bunny-shape
false
0
Circle -7566196 true true 59 56 186
Circle -7566196 true true 17 55 91
Circle -7566196 true true 194 197 46
Circle -7566196 true true 33 182 59
Circle -7566196 true true 219 104 47
Polygon -7566196 true true 56 56 111 5 119 27 91 68

rabbit-shape
false
0
Circle -7566196 true true 75 134 150
Polygon -7566196 true true 124 164 78 113 62 56 70 0 107 38 124 91
Polygon -7566196 true true 170 159 160 95 173 39 219 0 223 51 208 116

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
